/*
 * This file contains the user specified measurement functions for the different integration regions
 * for the correlated emission colour structures.
 * The recipe to derive the measurement functions F from the definition of the observable can be found 
 * in the manual.
 */

#include <Auxiliaries.h>
#include <Input_Common.h>
#include <Input_Parameters.h>
#include <cmath>
#include <iostream>

#ifdef BOOSTEDSOURCE_H_
using namespace boost::multiprecision;
using namespace boost::math;
#endif /* BOOSTEDSOURCE_H_ */

/*   __  __                                                    _      __                  _   _
 *  |  \/  |                                                  | |    / _|                | | (_)
 *  | \  / | ___  __ _ ___ _   _ _ __ ___ _ __ ___   ___ _ __ | |_  | |_ _   _ _ __   ___| |_ _  ___  _ __  ___
 *  | |\/| |/ _ \/ _` / __| | | | '__/ _ \ '_ ` _ \ / _ \ '_ \| __| |  _| | | | '_ \ / __| __| |/ _ \| '_ \/ __|
 *  | |  | |  __/ (_| \__ \ |_| | | |  __/ | | | | |  __/ | | | |_  | | | |_| | | | | (__| |_| | (_) | | | \__ \
 *  |_|  |_|\___|\__,_|___/\__,_|_|  \___|_| |_| |_|\___|_| |_|\__| |_|  \__,_|_| |_|\___|\__|_|\___/|_| |_|___/
 */

/*
 *    _____                    _       _           _
 *   / ____|                  | |     | |         | |
 *  | |     ___  _ __ _ __ ___| | __ _| |_ ___  __| |
 *  | |    / _ \| '__| '__/ _ \ |/ _` | __/ _ \/ _` |
 *  | |___| (_) | |  | | |  __/ | (_| | ||  __/ (_| |
 *   \_____\___/|_|  |_|  \___|_|\__,_|\__\___|\__,_|
 */


double FA(decimal v,decimal tl,decimal y,decimal tk,decimal u,decimal b) {
	// Please use these predefined variables a, tkl (=t_kl), ck (=cos theta_k) or tk (=(1-ck)/2), cl (=cos theta_l) or tl (=(1-cl)/2) together with b and y below
	decimal tkl=T(u,v);
	decimal ck=Ck(tk);
	decimal cl=Cl(tl);
	decimal a=A(u,v);
	decimal FA;

	/* Please input the C++ syntax measurement function F_A here. As a reminder: F_A you get by using the
	 * parametrisation for the correlated emission from the manual to the exponent of the full 
	 * measurement exponential, and discarding a few factors. 
	 *
	 * Don't forget to capture all zeroes and integrable divergences via if-clauses that set FA to a constant.
 	 */ 	
	bool th1u=((1.+a*b)*y-a*(a+b)>=0.);
	bool th2u=(b*pow(y,Beta/2.)*pow(1.+a*b,Beta/2.)*pow(a*(a+b)+y*(1.+a*b),1.+Beta/2.)-pow(a+b,Beta/2.)*pow(a+b+a*y*(1.+a*b),1.+Beta/2.)>=0);
	bool th3b1=((1.+a*b)*y-(a+b)*(a-2.+4.*tkl)>=0.);
	bool th3b2=(a*(1.+a*b)*y-(a+b)*(1-2.*a+4.*a*tkl)>=0.);
	bool th4b=(b*pow(a,Beta/2.)*pow(a*(a+b)+y*(1.+a*b),1.+Beta/2.)-pow(a+b+a*y*(1.+a*b),1.+Beta/2.)>=0.);

	decimal res1u=b*pow(a,-Beta/2.)*pow(y,Beta/2.)*pow(a+b,-1.-Beta/2.)*pow(a*(a+b)+y*(1.+a*b),1.+Beta/2.)*pow(1.+a*b,-1);
	decimal res2u=pow(a,-Beta/2.)*pow(1.+a*b,-1.-Beta/2.)*pow(a+b+a*y*(1.+a*b),1.+Beta/2.)*pow(a+b,-1);
	decimal res3b=pow(1.+y,1.+Beta/2.);
	decimal res4b=b*pow(1.+a*b,-1.-Beta/2.)*pow(a*(a+b)+y*(1.+a*b),1.+Beta/2.)*pow(a+b,-1);
	decimal res5b=pow(a,-Beta/2.)*pow(1.+a*b,-1.-Beta/2.)*pow(a+b+a*y*(1.+a*b),1.+Beta/2.)*pow(a+b,-1);
	
	if ( th1u ) {
		if ( th2u ) {
			FA=res1u;
		} else {
			FA=res2u;
		}
	} else if ( th3b1 && th3b2 ) {
			FA=res3b;
		} else if ( th4b ) {
			FA=res4b;
		} else {
			FA=res5b;
		}

	if (a<=0.) {
		FA=1.;
	}

	// Automated error message if a zero/divergence is not defused, please leave this as it is
	if (isnan(FA) || isinf(FA) || FA<=0.) {
		cerr<<"F_A error: zero, divergence or ambiguous expression with value FA="<<FA<<" at y="<<y<<" a="<<a<<" b="<<b<<" tkl="<<tkl<<" tl="<<tl<<" tk="<<tk<<endl;
		FA=1.;
	}
	return ReturnDouble(FA);
}

double FB(decimal v,decimal tl,decimal y,decimal tk,decimal u,decimal b) {
	// Please use these predefined variables a, tkl (=t_kl), ck (=cos theta_k) or tk (=(1-ck)/2), cl (=cos theta_l) or tl (=(1-cl)/2) together with b and y below
	decimal tkl=T(u,v);
	decimal ck=Ck(tk);
	decimal cl=Cl(tl);
	decimal a=A(u,v);
	decimal FB;

	/* Please input the C++ syntax measurement function F_B here. As a reminder: F_B you get by using the
	 * parametrisation for the correlated emission from the manual to the exponent of the full 
	 * measurement exponential, inverting ONE of EITHER y,a or b, and discarding a few factors. 
	 *
	 * Don't forget to capture all zeroes and integrable divergences via if-clauses that set FB to a constant.
 	 */ 
	bool th1b=((a+b)*y-a*(1.+a*b)>=0.);
	bool th2b=(b*pow(1.+a*b,Beta/2.)*pow(1.+a*b+a*y*(a+b),1.+Beta/2.)-pow(y,Beta/2.)*pow(a+b,Beta/2.)*pow(a*(1.+a*b)+y*(a+b),1.+Beta/2.)>=0.);
	bool th3b1=((a+b)*y-(1.+a*b)*(a-2.+4.*tkl)>=0.);
	bool th3b2=(a*(a+b)*y-(1.+a*b)*(1-2.*a+4.*a*tkl)>=0.);
	bool th4b=(b*pow(1.+a*b+a*y*(a+b),1.+Beta/2.)-pow(a,Beta/2.)*pow(a*(1.+a*b)+y*(a+b),1.+Beta/2.)>=0.);

	decimal res1b=b*pow(a,-Beta/2.)*pow(a+b,-1.-Beta/2.)*pow(y*a*(a+b)+1.+a*b,1.+Beta/2.)*pow(1.+a*b,-1);
	decimal res2b=pow(a,-Beta/2.)*pow(y,Beta/2.)*pow(1.+a*b,-1.-Beta/2.)*pow(y*(a+b)+a*(1.+a*b),1.+Beta/2.)*pow(a+b,-1);
	decimal res3b=pow(1.+y,1.+Beta/2.);
	decimal res4b=b*pow(a,-Beta/2.)*pow(a+b,-1.-Beta/2.)*pow(y*a*(a+b)+1.+a*b,1.+Beta/2.)*pow(1.+a*b,-1);
	decimal res5b=pow(a+b,-1.-Beta/2.)*pow(y*(a+b)+a*(1.+a*b),1.+Beta/2.)*pow(1.+a*b,-1);
	
	if ( th1b ) {
		if ( th2b ) {
			FB=res1b;
		} else {
			FB=res2b;
		}
	} else if ( th3b1 && th3b2 ) {
			FB=res3b;
		} else if ( th4b ) {
			FB=res4b;
		} else {
			FB=res5b;
		}
	
	if (a<=0.) {
		FB=1.;
	}

	// Automated error message if a zero/divergence is not defused, please leave this as it is
	if (isnan(FB) || isinf(FB) || FB<=0.) {
		cerr<<"F_B error: zero, divergence or ambiguous expression with value FB="<<FB<<" at y="<<y<<" a="<<a<<" b="<<b<<" tkl="<<tkl<<" tl="<<tl<<" tk="<<tk<<endl;
		FB=1.;
	}
	return ReturnDouble(FB);
}

/*This file is part of SoftSERVE.
 *
 *   SoftSERVE is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   SoftSERVE is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with SoftSERVE.  If not, see <http://www.gnu.org/licenses/>.
 */
