/*This file is part of SoftSERVE.
 *
 *   SoftSERVE is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   SoftSERVE is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with SoftSERVE.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * This file contains the difference structures arising from plus distributions tying together the Measurement functions FXi, plus distribution denominators 
 * and contributing remainder/numerator functions XCfEi for the 4D integration at all eps-orders of the Cf^2 structure, subregion b. 
 * Example: I4EM1kl ('I'ntegration over '4' dimensions, at 'E'ps to the 'M'inus '1'st order, third and fourth integrations over x'k' and x'l') contains various logs of FXi functions
 * evaluated at c=0,s5=0, i.e. with the two deltas from the c and s5 +-distributions, multiplied with the correct X functions' eps-powers to arrive at eps^-1. 
 */

#include <4Int.h>
#include <Auxiliaries.h>
#include <cmath>
#include <XFactor.h>
#include <Input_Common.h>

using namespace std;

double I4EM2kl(double xk, double xl, double s, double s6) {
	return 0; //no alpha-poles, but leading order is xi-indep
}
double I4EM1kl(double xk, double xl, double s, double s6) {
	return (pow(xk,-1)*pow(xl,-1)*(2*(log(FB1(0,0,t(s),0,tl(s6),0)) - log(FB1(0,yl(xl),t(s),0,tl(s6),0)) - log(FB1(yk(xk),0,t(s),0,tl(s6),0)) +
	          log(FB1(yk(xk),yl(xl),t(s),0,tl(s6),0)) + log(FB2(0,0,t(s),0,tl(s6),0)) - log(FB2(0,yl(xl),t(s),0,tl(s6),0)) -
	          log(FB2(yk(xk),0,t(s),0,tl(s6),0)) + log(FB2(yk(xk),yl(xl),t(s),0,tl(s6),0)))*XCfE0(s,0,s6) + XCfE1(0,0,s,0,s6) - XCfE1(0,xl,s,0,s6) -
	       XCfE1(xk,0,s,0,s6) + XCfE1(xk,xl,s,0,s6)))/4.;
}
double I4E0kl(double xk, double xl, double s, double s6) {
	return -(pow(xk,-1)*pow(xl,-1)*(-2*(mexp*n*log(xk)*(log(FB1(0,0,t(s),0,tl(s6),0)) - log(FB1(0,yl(xl),t(s),0,tl(s6),0)) - log(FB1(yk(xk),0,t(s),0,tl(s6),0)) + log(FB1(yk(xk),yl(xl),t(s),0,tl(s6),0)) +
            log(FB2(0,0,t(s),0,tl(s6),0)) - log(FB2(0,yl(xl),t(s),0,tl(s6),0)) - log(FB2(yk(xk),0,t(s),0,tl(s6),0)) + log(FB2(yk(xk),yl(xl),t(s),0,tl(s6),0))) +
         mexp*n*log(xl)*(log(FB1(0,0,t(s),0,tl(s6),0)) - log(FB1(0,yl(xl),t(s),0,tl(s6),0)) - log(FB1(yk(xk),0,t(s),0,tl(s6),0)) + log(FB1(yk(xk),yl(xl),t(s),0,tl(s6),0)) + log(FB2(0,0,t(s),0,tl(s6),0)) -
            log(FB2(0,yl(xl),t(s),0,tl(s6),0)) - log(FB2(yk(xk),0,t(s),0,tl(s6),0)) + log(FB2(yk(xk),yl(xl),t(s),0,tl(s6),0))) +
         2*(pow(log(FB1(0,0,t(s),0,tl(s6),0)),2) - pow(log(FB1(0,yl(xl),t(s),0,tl(s6),0)),2) - pow(log(FB1(yk(xk),0,t(s),0,tl(s6),0)),2) + pow(log(FB1(yk(xk),yl(xl),t(s),0,tl(s6),0)),2) +
            pow(log(FB2(0,0,t(s),0,tl(s6),0)),2) - pow(log(FB2(0,yl(xl),t(s),0,tl(s6),0)),2) - pow(log(FB2(yk(xk),0,t(s),0,tl(s6),0)),2) + pow(log(FB2(yk(xk),yl(xl),t(s),0,tl(s6),0)),2)))*XCfE0(s,0,s6) -
      (mexp*n*(log(xk) + log(xl)) + 2*(log(FB1(0,0,t(s),0,tl(s6),0)) + log(FB2(0,0,t(s),0,tl(s6),0))))*XCfE1(0,0,s,0,s6) + mexp*n*log(xk)*XCfE1(0,xl,s,0,s6) + mexp*n*log(xl)*XCfE1(0,xl,s,0,s6) +
      2*log(FB1(0,yl(xl),t(s),0,tl(s6),0))*XCfE1(0,xl,s,0,s6) + 2*log(FB2(0,yl(xl),t(s),0,tl(s6),0))*XCfE1(0,xl,s,0,s6) + mexp*n*log(xk)*XCfE1(xk,0,s,0,s6) + mexp*n*log(xl)*XCfE1(xk,0,s,0,s6) +
      2*log(FB1(yk(xk),0,t(s),0,tl(s6),0))*XCfE1(xk,0,s,0,s6) + 2*log(FB2(yk(xk),0,t(s),0,tl(s6),0))*XCfE1(xk,0,s,0,s6) -
      (mexp*n*(log(xk) + log(xl)) + 2*(log(FB1(yk(xk),yl(xl),t(s),0,tl(s6),0)) + log(FB2(yk(xk),yl(xl),t(s),0,tl(s6),0))))*XCfE1(xk,xl,s,0,s6) - XCfE2(0,0,s,0,s6) + XCfE2(0,xl,s,0,s6) + XCfE2(xk,0,s,0,s6) -
      XCfE2(xk,xl,s,0,s6)))/4.;
}

double I4EM2kc(double c, double xk, double s, double s6) {
	return 0; //one alpha pole, which can be cancelled, but not converted
}
double I4EM1kc(double c, double xk, double s, double s6) {
	return -2*(log(FB1(0,0,t(s),0,tl(s6),0)) - log(FB1(0,0,t(s),0,tl(s6),b(c))) - log(FB1(yk(xk),0,t(s),0,tl(s6),0)) +
		     log(FB1(yk(xk),0,t(s),0,tl(s6),b(c))) + log(FB2(0,0,t(s),0,tl(s6),0)) - log(FB2(0,0,t(s),0,tl(s6),b(c))) -
		     log(FB2(yk(xk),0,t(s),0,tl(s6),0)) + log(FB2(yk(xk),0,t(s),0,tl(s6),b(c))))*pow(c,-1)*pow(mexp,-1)*pow(n,-1)*pow(xk,-1)*XCfE0(s,0,s6);
}
double I4E0kc(double c, double xk, double s, double s6) {
	return 2*pow(c,-1)*pow(mexp,-1)*pow(n,-1)*pow(xk,-1)*((mexp*n*log(xk)*(-log(FB1(0,0,t(s),0,tl(s6),0)) + log(FB1(0,0,t(s),0,tl(s6),b(c))) +
	           log(FB1(yk(xk),0,t(s),0,tl(s6),0)) - log(FB1(yk(xk),0,t(s),0,tl(s6),b(c))) - log(FB2(0,0,t(s),0,tl(s6),0)) +
	           log(FB2(0,0,t(s),0,tl(s6),b(c))) + log(FB2(yk(xk),0,t(s),0,tl(s6),0)) - log(FB2(yk(xk),0,t(s),0,tl(s6),b(c)))) +
	        4*log(c)*(log(FB1(0,0,t(s),0,tl(s6),0)) - log(FB1(0,0,t(s),0,tl(s6),b(c))) - log(FB1(yk(xk),0,t(s),0,tl(s6),0)) +
	           log(FB1(yk(xk),0,t(s),0,tl(s6),b(c))) + log(FB2(0,0,t(s),0,tl(s6),0)) - log(FB2(0,0,t(s),0,tl(s6),b(c))) -
	           log(FB2(yk(xk),0,t(s),0,tl(s6),0)) + log(FB2(yk(xk),0,t(s),0,tl(s6),b(c)))) -
	        2*(pow(log(FB1(0,0,t(s),0,tl(s6),0)),2) - pow(log(FB1(0,0,t(s),0,tl(s6),b(c))),2) - pow(log(FB1(yk(xk),0,t(s),0,tl(s6),0)),2) +
	           pow(log(FB1(yk(xk),0,t(s),0,tl(s6),b(c))),2) + pow(log(FB2(0,0,t(s),0,tl(s6),0)),2) - pow(log(FB2(0,0,t(s),0,tl(s6),b(c))),2) -
	           pow(log(FB2(yk(xk),0,t(s),0,tl(s6),0)),2) + pow(log(FB2(yk(xk),0,t(s),0,tl(s6),b(c))),2)))*XCfE0(s,0,s6) +
	     (-log(FB1(0,0,t(s),0,tl(s6),0)) + log(FB1(0,0,t(s),0,tl(s6),b(c))) - log(FB2(0,0,t(s),0,tl(s6),0)) + log(FB2(0,0,t(s),0,tl(s6),b(c))))*
	      XCfE1(0,0,s,0,s6) + (log(FB1(yk(xk),0,t(s),0,tl(s6),0)) - log(FB1(yk(xk),0,t(s),0,tl(s6),b(c))) + log(FB2(yk(xk),0,t(s),0,tl(s6),0)) -
	        log(FB2(yk(xk),0,t(s),0,tl(s6),b(c))))*XCfE1(xk,0,s,0,s6));
}

double I4EM2ks(double xk, double s, double s5, double s6) {
	return 0; // one alpha pole, but leading order is xk-indep
}
double I4EM1ks(double xk, double s, double s5, double s6) {
	return (pow(mexp,-1)*pow(n,-1)*pow(s5,-1)*pow(xk,-1)*(2*(-log(FB1(0,0,t(s),0,tl(s6),0)) + log(FB1(yk(xk),0,t(s),0,tl(s6),0)) -
	          log(FB2(0,0,t(s),0,tl(s6),0)) + log(FB2(yk(xk),0,t(s),0,tl(s6),0)))*XCfE0(s,0,s6) +
	       2*(log(FB1(0,0,t(s),t5(s5),tl(s6),0)) - log(FB1(yk(xk),0,t(s),t5(s5),tl(s6),0)) + log(FB2(0,0,t(s),t5(s5),tl(s6),0)) -
	          log(FB2(yk(xk),0,t(s),t5(s5),tl(s6),0)))*XCfE0(s,s5,s6) - XCfE1(0,0,s,0,s6) + XCfE1(0,0,s,s5,s6) + XCfE1(xk,0,s,0,s6) -
	       XCfE1(xk,0,s,s5,s6)))/2.;
}
double I4E0ks(double xk, double s, double s5, double s6) {
	return (pow(mexp,-1)*pow(n,-1)*pow(s5,-1)*pow(xk,-1)*(2*(2*log(s5)*(log(FB1(0,0,t(s),0,tl(s6),0)) - log(FB1(yk(xk),0,t(s),0,tl(s6),0)) +
            log(FB2(0,0,t(s),0,tl(s6),0)) - log(FB2(yk(xk),0,t(s),0,tl(s6),0))) +
         mexp*n*log(xk)*(-log(FB1(0,0,t(s),0,tl(s6),0)) + log(FB1(yk(xk),0,t(s),0,tl(s6),0)) - log(FB2(0,0,t(s),0,tl(s6),0)) +
            log(FB2(yk(xk),0,t(s),0,tl(s6),0))) - 2*(pow(log(FB1(0,0,t(s),0,tl(s6),0)),2) - pow(log(FB1(yk(xk),0,t(s),0,tl(s6),0)),2) +
            pow(log(FB2(0,0,t(s),0,tl(s6),0)),2) - pow(log(FB2(yk(xk),0,t(s),0,tl(s6),0)),2)))*XCfE0(s,0,s6) +
      2*(mexp*n*log(xk)*(log(FB1(0,0,t(s),t5(s5),tl(s6),0)) - log(FB1(yk(xk),0,t(s),t5(s5),tl(s6),0)) + log(FB2(0,0,t(s),t5(s5),tl(s6),0)) -
            log(FB2(yk(xk),0,t(s),t5(s5),tl(s6),0))) + 2*log(s5)*
          (-log(FB1(0,0,t(s),t5(s5),tl(s6),0)) + log(FB1(yk(xk),0,t(s),t5(s5),tl(s6),0)) - log(FB2(0,0,t(s),t5(s5),tl(s6),0)) +
            log(FB2(yk(xk),0,t(s),t5(s5),tl(s6),0))) + 2*(pow(log(FB1(0,0,t(s),t5(s5),tl(s6),0)),2) -
            pow(log(FB1(yk(xk),0,t(s),t5(s5),tl(s6),0)),2) + pow(log(FB2(0,0,t(s),t5(s5),tl(s6),0)),2) -
            pow(log(FB2(yk(xk),0,t(s),t5(s5),tl(s6),0)),2)))*XCfE0(s,s5,s6) + 2*log(s5)*XCfE1(0,0,s,0,s6) - mexp*n*log(xk)*XCfE1(0,0,s,0,s6) -
      2*log(FB1(0,0,t(s),0,tl(s6),0))*XCfE1(0,0,s,0,s6) - 2*log(FB2(0,0,t(s),0,tl(s6),0))*XCfE1(0,0,s,0,s6) - 2*log(s5)*XCfE1(0,0,s,s5,s6) +
      mexp*n*log(xk)*XCfE1(0,0,s,s5,s6) + 2*log(FB1(0,0,t(s),t5(s5),tl(s6),0))*XCfE1(0,0,s,s5,s6) +
      2*log(FB2(0,0,t(s),t5(s5),tl(s6),0))*XCfE1(0,0,s,s5,s6) - 2*log(s5)*XCfE1(xk,0,s,0,s6) + mexp*n*log(xk)*XCfE1(xk,0,s,0,s6) +
      2*log(FB1(yk(xk),0,t(s),0,tl(s6),0))*XCfE1(xk,0,s,0,s6) + 2*log(FB2(yk(xk),0,t(s),0,tl(s6),0))*XCfE1(xk,0,s,0,s6) +
      (2*log(s5) - mexp*n*log(xk) - 2*(log(FB1(yk(xk),0,t(s),t5(s5),tl(s6),0)) + log(FB2(yk(xk),0,t(s),t5(s5),tl(s6),0))))*XCfE1(xk,0,s,s5,s6) -
      XCfE2(0,0,s,0,s6) + XCfE2(0,0,s,s5,s6) + XCfE2(xk,0,s,0,s6) - XCfE2(xk,0,s,s5,s6)))/2.;
}

double I4EM2lc(double c, double xl, double s, double s6) {
	return 0; //leading order is xl-indep
}
double I4EM1lc(double c, double xl, double s, double s6) {
	return -2*(log(FB1(0,0,t(s),0,tl(s6),0)) - log(FB1(0,0,t(s),0,tl(s6),b(c))) - log(FB1(0,yl(xl),t(s),0,tl(s6),0)) +
		     log(FB1(0,yl(xl),t(s),0,tl(s6),b(c))) + log(FB2(0,0,t(s),0,tl(s6),0)) - log(FB2(0,0,t(s),0,tl(s6),b(c))) -
		     log(FB2(0,yl(xl),t(s),0,tl(s6),0)) + log(FB2(0,yl(xl),t(s),0,tl(s6),b(c))))*pow(c,-1)*pow(mexp,-1)*pow(n,-1)*pow(xl,-1)*XCfE0(s,0,s6);
}
double I4E0lc(double c, double xl, double s, double s6) {
	return 2*pow(c,-1)*pow(mexp,-1)*pow(n,-1)*pow(xl,-1)*((mexp*n*log(xl)*(-log(FB1(0,0,t(s),0,tl(s6),0)) + log(FB1(0,0,t(s),0,tl(s6),b(c))) +
	           log(FB1(0,yl(xl),t(s),0,tl(s6),0)) - log(FB1(0,yl(xl),t(s),0,tl(s6),b(c))) - log(FB2(0,0,t(s),0,tl(s6),0)) +
	           log(FB2(0,0,t(s),0,tl(s6),b(c))) + log(FB2(0,yl(xl),t(s),0,tl(s6),0)) - log(FB2(0,yl(xl),t(s),0,tl(s6),b(c)))) +
	        4*log(c)*(log(FB1(0,0,t(s),0,tl(s6),0)) - log(FB1(0,0,t(s),0,tl(s6),b(c))) - log(FB1(0,yl(xl),t(s),0,tl(s6),0)) +
	           log(FB1(0,yl(xl),t(s),0,tl(s6),b(c))) + log(FB2(0,0,t(s),0,tl(s6),0)) - log(FB2(0,0,t(s),0,tl(s6),b(c))) -
	           log(FB2(0,yl(xl),t(s),0,tl(s6),0)) + log(FB2(0,yl(xl),t(s),0,tl(s6),b(c)))) -
	        2*(pow(log(FB1(0,0,t(s),0,tl(s6),0)),2) - pow(log(FB1(0,0,t(s),0,tl(s6),b(c))),2) - pow(log(FB1(0,yl(xl),t(s),0,tl(s6),0)),2) +
	           pow(log(FB1(0,yl(xl),t(s),0,tl(s6),b(c))),2) + pow(log(FB2(0,0,t(s),0,tl(s6),0)),2) - pow(log(FB2(0,0,t(s),0,tl(s6),b(c))),2) -
	           pow(log(FB2(0,yl(xl),t(s),0,tl(s6),0)),2) + pow(log(FB2(0,yl(xl),t(s),0,tl(s6),b(c))),2)))*XCfE0(s,0,s6) +
	     (-log(FB1(0,0,t(s),0,tl(s6),0)) + log(FB1(0,0,t(s),0,tl(s6),b(c))) - log(FB2(0,0,t(s),0,tl(s6),0)) + log(FB2(0,0,t(s),0,tl(s6),b(c))))*
	      XCfE1(0,0,s,0,s6) + (log(FB1(0,yl(xl),t(s),0,tl(s6),0)) - log(FB1(0,yl(xl),t(s),0,tl(s6),b(c))) + log(FB2(0,yl(xl),t(s),0,tl(s6),0)) -
	        log(FB2(0,yl(xl),t(s),0,tl(s6),b(c))))*XCfE1(0,xl,s,0,s6));
}

double I4EM2ls(double xl, double s, double s5, double s6) {
	return 0; //leading order is xl-indep
}
double I4EM1ls(double xl, double s, double s5, double s6) {
	return (pow(mexp,-1)*pow(n,-1)*pow(s5,-1)*pow(xl,-1)*(2*(-log(FB1(0,0,t(s),0,tl(s6),0)) + log(FB1(0,yl(xl),t(s),0,tl(s6),0)) -
	          log(FB2(0,0,t(s),0,tl(s6),0)) + log(FB2(0,yl(xl),t(s),0,tl(s6),0)))*XCfE0(s,0,s6) +
	       2*(log(FB1(0,0,t(s),t5(s5),tl(s6),0)) - log(FB1(0,yl(xl),t(s),t5(s5),tl(s6),0)) + log(FB2(0,0,t(s),t5(s5),tl(s6),0)) -
	          log(FB2(0,yl(xl),t(s),t5(s5),tl(s6),0)))*XCfE0(s,s5,s6) - XCfE1(0,0,s,0,s6) + XCfE1(0,0,s,s5,s6) + XCfE1(0,xl,s,0,s6) -
	       XCfE1(0,xl,s,s5,s6)))/2.;
}
double I4E0ls(double xl, double s, double s5, double s6) {
	return (pow(mexp,-1)*pow(n,-1)*pow(s5,-1)*pow(xl,-1)*(2*(2*log(s5)*(log(FB1(0,0,t(s),0,tl(s6),0)) - log(FB1(0,yl(xl),t(s),0,tl(s6),0)) +
            log(FB2(0,0,t(s),0,tl(s6),0)) - log(FB2(0,yl(xl),t(s),0,tl(s6),0))) +
         mexp*n*log(xl)*(-log(FB1(0,0,t(s),0,tl(s6),0)) + log(FB1(0,yl(xl),t(s),0,tl(s6),0)) - log(FB2(0,0,t(s),0,tl(s6),0)) +
            log(FB2(0,yl(xl),t(s),0,tl(s6),0))) - 2*(pow(log(FB1(0,0,t(s),0,tl(s6),0)),2) - pow(log(FB1(0,yl(xl),t(s),0,tl(s6),0)),2) +
            pow(log(FB2(0,0,t(s),0,tl(s6),0)),2) - pow(log(FB2(0,yl(xl),t(s),0,tl(s6),0)),2)))*XCfE0(s,0,s6) +
      2*(mexp*n*log(xl)*(log(FB1(0,0,t(s),t5(s5),tl(s6),0)) - log(FB1(0,yl(xl),t(s),t5(s5),tl(s6),0)) + log(FB2(0,0,t(s),t5(s5),tl(s6),0)) -
            log(FB2(0,yl(xl),t(s),t5(s5),tl(s6),0))) + 2*log(s5)*
          (-log(FB1(0,0,t(s),t5(s5),tl(s6),0)) + log(FB1(0,yl(xl),t(s),t5(s5),tl(s6),0)) - log(FB2(0,0,t(s),t5(s5),tl(s6),0)) +
            log(FB2(0,yl(xl),t(s),t5(s5),tl(s6),0))) + 2*(pow(log(FB1(0,0,t(s),t5(s5),tl(s6),0)),2) -
            pow(log(FB1(0,yl(xl),t(s),t5(s5),tl(s6),0)),2) + pow(log(FB2(0,0,t(s),t5(s5),tl(s6),0)),2) -
            pow(log(FB2(0,yl(xl),t(s),t5(s5),tl(s6),0)),2)))*XCfE0(s,s5,s6) + 2*log(s5)*XCfE1(0,0,s,0,s6) - mexp*n*log(xl)*XCfE1(0,0,s,0,s6) -
      2*log(FB1(0,0,t(s),0,tl(s6),0))*XCfE1(0,0,s,0,s6) - 2*log(FB2(0,0,t(s),0,tl(s6),0))*XCfE1(0,0,s,0,s6) - 2*log(s5)*XCfE1(0,0,s,s5,s6) +
      mexp*n*log(xl)*XCfE1(0,0,s,s5,s6) + 2*log(FB1(0,0,t(s),t5(s5),tl(s6),0))*XCfE1(0,0,s,s5,s6) +
      2*log(FB2(0,0,t(s),t5(s5),tl(s6),0))*XCfE1(0,0,s,s5,s6) - 2*log(s5)*XCfE1(0,xl,s,0,s6) + mexp*n*log(xl)*XCfE1(0,xl,s,0,s6) +
      2*log(FB1(0,yl(xl),t(s),0,tl(s6),0))*XCfE1(0,xl,s,0,s6) + 2*log(FB2(0,yl(xl),t(s),0,tl(s6),0))*XCfE1(0,xl,s,0,s6) +
      (2*log(s5) - mexp*n*log(xl) - 2*(log(FB1(0,yl(xl),t(s),t5(s5),tl(s6),0)) + log(FB2(0,yl(xl),t(s),t5(s5),tl(s6),0))))*XCfE1(0,xl,s,s5,s6) -
      XCfE2(0,0,s,0,s6) + XCfE2(0,0,s,s5,s6) + XCfE2(0,xl,s,0,s6) - XCfE2(0,xl,s,s5,s6)))/2.;
}

double I4EM2cs(double c, double s, double s5, double s6) {
	return 0; //leading order c-indep
}
double I4EM1cs(double c, double s, double s5, double s6) {
	return 4*pow(c,-1)*pow(mexp,-2)*pow(n,-2)*pow(s5,-1)*((log(FB1(0,0,t(s),0,tl(s6),0)) - log(FB1(0,0,t(s),0,tl(s6),b(c))) + log(FB2(0,0,t(s),0,tl(s6),0)) -
	        log(FB2(0,0,t(s),0,tl(s6),b(c))))*XCfE0(s,0,s6) + (-log(FB1(0,0,t(s),t5(s5),tl(s6),0)) + log(FB1(0,0,t(s),t5(s5),tl(s6),b(c))) -
	        log(FB2(0,0,t(s),t5(s5),tl(s6),0)) + log(FB2(0,0,t(s),t5(s5),tl(s6),b(c))))*XCfE0(s,s5,s6));
}
double I4E0cs(double c, double s, double s5, double s6) {
	return 4*pow(c,-1)*pow(mexp,-2)*pow(n,-2)*pow(s5,-1)*(-2*(2*log(c)*(log(FB1(0,0,t(s),0,tl(s6),0)) - log(FB1(0,0,t(s),0,tl(s6),b(c))) +
	           log(FB2(0,0,t(s),0,tl(s6),0)) - log(FB2(0,0,t(s),0,tl(s6),b(c)))) +
	        log(s5)*(log(FB1(0,0,t(s),0,tl(s6),0)) - log(FB1(0,0,t(s),0,tl(s6),b(c))) + log(FB2(0,0,t(s),0,tl(s6),0)) -
	           log(FB2(0,0,t(s),0,tl(s6),b(c)))) - pow(log(FB1(0,0,t(s),0,tl(s6),0)),2) + pow(log(FB1(0,0,t(s),0,tl(s6),b(c))),2) -
	        pow(log(FB2(0,0,t(s),0,tl(s6),0)),2) + pow(log(FB2(0,0,t(s),0,tl(s6),b(c))),2))*XCfE0(s,0,s6) +
	     2*(2*log(c)*(log(FB1(0,0,t(s),t5(s5),tl(s6),0)) - log(FB1(0,0,t(s),t5(s5),tl(s6),b(c))) + log(FB2(0,0,t(s),t5(s5),tl(s6),0)) -
	           log(FB2(0,0,t(s),t5(s5),tl(s6),b(c)))) + log(s5)*
	         (log(FB1(0,0,t(s),t5(s5),tl(s6),0)) - log(FB1(0,0,t(s),t5(s5),tl(s6),b(c))) + log(FB2(0,0,t(s),t5(s5),tl(s6),0)) -
	           log(FB2(0,0,t(s),t5(s5),tl(s6),b(c)))) - pow(log(FB1(0,0,t(s),t5(s5),tl(s6),0)),2) + pow(log(FB1(0,0,t(s),t5(s5),tl(s6),b(c))),2) -
	        pow(log(FB2(0,0,t(s),t5(s5),tl(s6),0)),2) + pow(log(FB2(0,0,t(s),t5(s5),tl(s6),b(c))),2))*XCfE0(s,s5,s6) +
	     (log(FB1(0,0,t(s),0,tl(s6),0)) - log(FB1(0,0,t(s),0,tl(s6),b(c))) + log(FB2(0,0,t(s),0,tl(s6),0)) - log(FB2(0,0,t(s),0,tl(s6),b(c))))*
	      XCfE1(0,0,s,0,s6) + (-log(FB1(0,0,t(s),t5(s5),tl(s6),0)) + log(FB1(0,0,t(s),t5(s5),tl(s6),b(c))) - log(FB2(0,0,t(s),t5(s5),tl(s6),0)) +
	        log(FB2(0,0,t(s),t5(s5),tl(s6),b(c))))*XCfE1(0,0,s,s5,s6));
}

