/*This file is part of SoftSERVE.
 *
 *   SoftSERVE is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   SoftSERVE is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with SoftSERVE.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * This file contains the eps-expansion of all functions and expressions in the Cf^2 colour structure, subregion b that are neither +-distributions or measurement functions 
 * (i.e. mainly structures from Jacobians and matrix element numerators, and non-divergent denominator structures), collected into one container X. The 
 * eps-expansion orders of this container appear in the differences generated by +-distributions and multiply the Logs of the measurement functions. They are 
 * constructed by substitution of variables in such a way to vanish at all non-critical boundaries (i.e. boundaries that have diverging logs attached, like 
 * Log(s6) at 0 or Log(1-s) at 1. As C++ sees x*Log(x) as 0*NaN, these have to be caught via if-clauses for all but the first eps-order.
 * The notation is such that XCfEi is the eps^i order of the X-container for colour structure Cf^2, subregion b.
 */

#include <XFactor.h>
#include <cmath>
#include <Auxiliaries.h>
#include <Input_Common.h>
using namespace std;

double XCfE0(double s,double s5, double s6) {
	return (1 - s)*s*(1 + s)*(1 - s6)*s6*(1 + s6)*(1 + pow(s,2))*(1 + pow(s6,2))*pow(2 - pow(s,4),-0.5)*pow(2 - 2*pow(s,4) + pow(s,8),-0.5)*
			   pow(2 - pow(s5,2),-1)*pow(2 - pow(s6,4),-0.5)*pow(2 - 2*pow(s6,4) + pow(s6,8),-0.5);
}

double XCfE1(double xk,double xl,double s,double s5, double s6) {
	if(s6>=1. || s6<=0. || s<=0. || s>=1.) {
		return 0;
	} else {
		return -((1 - s)*s*(1 + s)*(1 - s6)*s6*(1 + s6)*(4*log(1 - s) + 4*log(s) + 4*log(1 + s) + 4*log(1 - s6) + 4*log(s6) + 4*log(1 + s6) +
			       4*log(1 + pow(s,2)) + log(2 - pow(s,4)) + log(2 - 2*pow(s,4) + pow(s,8)) + log(2 - pow(s5,2)) + 4*log(1 + pow(s6,2)) +
			       log(2 - pow(s6,4)) + log(2 - 2*pow(s6,4) + pow(s6,8)) - 2*n*log(1 + pow(xk,mexp)) - 2*n*log(1 + pow(xl,mexp)))*(1 + pow(s,2))*(1 + pow(s6,2))*
			     pow(2 - pow(s,4),-0.5)*pow(2 - 2*pow(s,4) + pow(s,8),-0.5)*pow(2 - pow(s5,2),-1)*pow(2 - pow(s6,4),-0.5)*
			     pow(2 - 2*pow(s6,4) + pow(s6,8),-0.5));
	}
}

double XCfE2(double xk,double xl,double s,double s5, double s6) {
	if(s6>=1. || s6<=0. || s<=0. || s>=1.) {
		return 0;
	} else {
		return ((1 - s)*s*(1 + s)*(1 - s6)*s6*(1 + s6)*(1 + pow(s,2))*(1 + pow(s6,2))*
			     pow(4*log(1 - s) + 4*log(s) + 4*log(1 + s) + 4*log(1 - s6) + 4*log(s6) + 4*log(1 + s6) + 4*log(1 + pow(s,2)) + log(2 - pow(s,4)) +
			       log(2 - 2*pow(s,4) + pow(s,8)) + log(2 - pow(s5,2)) + 4*log(1 + pow(s6,2)) + log(2 - pow(s6,4)) + log(2 - 2*pow(s6,4) + pow(s6,8)) -
			       2*n*log(1 + pow(xk,mexp)) - 2*n*log(1 + pow(xl,mexp)),2)*pow(2 - pow(s,4),-0.5)*pow(2 - 2*pow(s,4) + pow(s,8),-0.5)*pow(2 - pow(s5,2),-1)*
			     pow(2 - pow(s6,4),-0.5)*pow(2 - 2*pow(s6,4) + pow(s6,8),-0.5))/2.;
	}
}

double XCfE3(double xk,double xl,double s,double s5, double s6) {
	if(s6>=1. || s6<=0. || s<=0. || s>=1.) {
		return 0;
	} else {
		return -((1 - s)*s*(1 + s)*(1 - s6)*s6*(1 + s6)*(1 + pow(s,2))*(1 + pow(s6,2))*
			      pow(4*log(1 - s) + 4*log(s) + 4*log(1 + s) + 4*log(1 - s6) + 4*log(s6) + 4*log(1 + s6) + 4*log(1 + pow(s,2)) + log(2 - pow(s,4)) +
			        log(2 - 2*pow(s,4) + pow(s,8)) + log(2 - pow(s5,2)) + 4*log(1 + pow(s6,2)) + log(2 - pow(s6,4)) + log(2 - 2*pow(s6,4) + pow(s6,8)) -
			        2*n*log(1 + pow(xk,mexp)) - 2*n*log(1 + pow(xl,mexp)),3)*pow(2 - pow(s,4),-0.5)*pow(2 - 2*pow(s,4) + pow(s,8),-0.5)*pow(2 - pow(s5,2),-1)*
			      pow(2 - pow(s6,4),-0.5)*pow(2 - 2*pow(s6,4) + pow(s6,8),-0.5))/6.;
	}
}

double XCfE4(double xk,double xl,double s,double s5, double s6) {
	if(s6>=1. || s6<=0. || s<=0. || s>=1.) {
		return 0;
	} else {
		return ((1 - s)*s*(1 + s)*(1 - s6)*s6*(1 + s6)*(1 + pow(s,2))*(1 + pow(s6,2))*
			     pow(-4*log(1 - s) - 4*log(s) - 4*log(1 + s) - 4*log(1 - s6) - 4*log(s6) - 4*log(1 + s6) - 4*log(1 + pow(s,2)) - log(2 - pow(s,4)) -
			       log(2 - 2*pow(s,4) + pow(s,8)) - log(2 - pow(s5,2)) - 4*log(1 + pow(s6,2)) - log(2 - pow(s6,4)) - log(2 - 2*pow(s6,4) + pow(s6,8)) +
			       2*n*log(1 + pow(xk,mexp)) + 2*n*log(1 + pow(xl,mexp)),4)*pow(2 - pow(s,4),-0.5)*pow(2 - 2*pow(s,4) + pow(s,8),-0.5)*pow(2 - pow(s5,2),-1)*
			     pow(2 - pow(s6,4),-0.5)*pow(2 - 2*pow(s6,4) + pow(s6,8),-0.5))/24.;
	}
}

