/*
 * This file contains the difference structures arising from plus distributions tying together the Measurement functions FXi, plus distribution denominators 
 * and contributing remainder/numerator functions XCAij for the 3D integration at all eps-orders and the alpha^0 order of the Rest-CA structure. 
 * Example: CAI3EM2s ('CA' structure, 'I'ntegration over '3' dimensions, at 'E'ps to the 'M'inus '2'nd and alpha^0 order, third integration over 's'5) 
 * contains various differences of logs of FXi functions evaluated at z=0,c=0,x=0, i.e. with deltas from all +-distributions except the t5/s5 one, multiplied 
 * with the correct XCA functions to arrive at eps^-2 alpha^0. 
 */

#include <3IntCA.h>
#include <Auxiliaries.h>
#include <cmath>
#include <XFactorCA.h>
#include <Input_Common.h>
using namespace std;

//third integration is over t5 (s5 after rescaling)

double CAI3EM3s(double w, double s6, double s5) {
	return (pow(s5,-1)*(XCA00(w,s6,0,0,0) - XCA00(w,s6,s5,0,0)))/8.;
}

double CAI3EM2s(double w, double s6, double s5) {
	return (pow(s5,-1)*(-2*log(s5)*XCA00(w,s6,0,0,0) + 2*log(s5)*XCA00(w,s6,s5,0,0) + XCA01(w,s6,0,0,0) - XCA01(w,s6,s5,0,0)))/8.;
}

double CAI3EM1s(double w, double s6, double s5) {
	return (pow(s5,-1)*(2*(pow(log(s5),2) - pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) - pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) - pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) - pow(log(FB2(v(w),tl(s6),0,0,0,0)),2))*
        XCA00(w,s6,0,0,0) + 2*(-pow(log(s5),2) + pow(log(FA1(v(w),tl(s6),0,t5(s5),0,0)),2) + pow(log(FA2(v(w),tl(s6),0,t5(s5),0,0)),2) + pow(log(FB1(v(w),tl(s6),0,t5(s5),0,0)),2) + 
          pow(log(FB2(v(w),tl(s6),0,t5(s5),0,0)),2))*XCA00(w,s6,s5,0,0) - 2*log(s5)*XCA01(w,s6,0,0,0) + 2*log(s5)*XCA01(w,s6,s5,0,0) + XCA02(w,s6,0,0,0) - XCA02(w,s6,s5,0,0)))/8.;
}

double CAI3E0s(double w, double s6, double s5) {
	return (pow(s5,-1)*(-4*(pow(log(s5),3) - 3*log(s5)*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) + 
             pow(log(FB2(v(w),tl(s6),0,0,0,0)),2)) + 4*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),3) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),3) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),3) + 
             pow(log(FB2(v(w),tl(s6),0,0,0,0)),3)))*XCA00(w,s6,0,0,0) + 4*(pow(log(s5),3) - 
          3*log(s5)*(pow(log(FA1(v(w),tl(s6),0,t5(s5),0,0)),2) + pow(log(FA2(v(w),tl(s6),0,t5(s5),0,0)),2) + pow(log(FB1(v(w),tl(s6),0,t5(s5),0,0)),2) + 
             pow(log(FB2(v(w),tl(s6),0,t5(s5),0,0)),2)) + 4*(pow(log(FA1(v(w),tl(s6),0,t5(s5),0,0)),3) + pow(log(FA2(v(w),tl(s6),0,t5(s5),0,0)),3) + 
             pow(log(FB1(v(w),tl(s6),0,t5(s5),0,0)),3) + pow(log(FB2(v(w),tl(s6),0,t5(s5),0,0)),3)))*XCA00(w,s6,s5,0,0) + 
       3*(2*(pow(log(s5),2) - pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) - pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) - pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) - pow(log(FB2(v(w),tl(s6),0,0,0,0)),2))*
           XCA01(w,s6,0,0,0) + 2*(-pow(log(s5),2) + pow(log(FA1(v(w),tl(s6),0,t5(s5),0,0)),2) + pow(log(FA2(v(w),tl(s6),0,t5(s5),0,0)),2) + pow(log(FB1(v(w),tl(s6),0,t5(s5),0,0)),2) + 
             pow(log(FB2(v(w),tl(s6),0,t5(s5),0,0)),2))*XCA01(w,s6,s5,0,0) - 2*log(s5)*XCA02(w,s6,0,0,0) + 2*log(s5)*XCA02(w,s6,s5,0,0) + XCA03(w,s6,0,0,0) - XCA03(w,s6,s5,0,0))))/24.;
}

//third integration is over u (z after rescaling)

double CAI3EM3z(double w, double s6, double z) {
	return (pow(z,-1)*(XCA00(w,s6,0,0,0) - XCA00(w,s6,0,z,0)))/4.;
}

double CAI3EM2z(double w, double s6, double z) {
	return (pow(z,-1)*(-4*log(z)*XCA00(w,s6,0,0,0) + (4*log(z) + log(RegulA(0,z,w)*RegulB(0,z,w)))*XCA00(w,s6,0,z,0) + XCA01(w,s6,0,0,0) - XCA01(w,s6,0,z,0)))/4.;
}

double CAI3EM1z(double w, double s6, double z) {
	return (pow(z,-1)*((8*pow(log(z),2) - 2*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),2)))*
        XCA00(w,s6,0,0,0) + 2*(log(FA1(v(w),tl(s6),0,0,u(z),0)*FA2(v(w),tl(s6),0,0,u(z),0))*log(RegulA(0,z,w)) + 
          log(FB1(v(w),tl(s6),0,0,u(z),0)*FB2(v(w),tl(s6),0,0,u(z),0))*log(RegulB(0,z,w)) - 2*log(z)*log(RegulA(0,z,w)*RegulB(0,z,w)) - 4*pow(log(z),2) + 
          pow(log(FA1(v(w),tl(s6),0,0,u(z),0)),2) + pow(log(FA2(v(w),tl(s6),0,0,u(z),0)),2) + pow(log(FB1(v(w),tl(s6),0,0,u(z),0)),2) + pow(log(FB2(v(w),tl(s6),0,0,u(z),0)),2))*
        XCA00(w,s6,0,z,0) - 4*log(z)*XCA01(w,s6,0,0,0) + 4*log(z)*XCA01(w,s6,0,z,0) + log(RegulA(0,z,w)*RegulB(0,z,w))*XCA01(w,s6,0,z,0) + XCA02(w,s6,0,0,0) - XCA02(w,s6,0,z,0)))/4.;
}

double CAI3E0z(double w, double s6, double z) {
	return (pow(z,-1)*(-8*(4*pow(log(z),3) - 3*log(z)*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) + 
             pow(log(FB2(v(w),tl(s6),0,0,0,0)),2)) + 2*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),3) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),3) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),3) + 
             pow(log(FB2(v(w),tl(s6),0,0,0,0)),3)))*XCA00(w,s6,0,0,0) + 4*(6*log(RegulA(0,z,w)*RegulB(0,z,w))*pow(log(z),2) + 8*pow(log(z),3) + 
          3*log(RegulA(0,z,w))*pow(log(FA1(v(w),tl(s6),0,0,u(z),0)),2) + 4*pow(log(FA1(v(w),tl(s6),0,0,u(z),0)),3) + 3*log(RegulA(0,z,w))*pow(log(FA2(v(w),tl(s6),0,0,u(z),0)),2) + 
          4*pow(log(FA2(v(w),tl(s6),0,0,u(z),0)),3) + 3*log(RegulB(0,z,w))*pow(log(FB1(v(w),tl(s6),0,0,u(z),0)),2) + 4*pow(log(FB1(v(w),tl(s6),0,0,u(z),0)),3) + 
          3*log(RegulB(0,z,w))*pow(log(FB2(v(w),tl(s6),0,0,u(z),0)),2) - 6*log(z)*
           (log(FA1(v(w),tl(s6),0,0,u(z),0)*FA2(v(w),tl(s6),0,0,u(z),0))*log(RegulA(0,z,w)) + log(FB1(v(w),tl(s6),0,0,u(z),0)*FB2(v(w),tl(s6),0,0,u(z),0))*log(RegulB(0,z,w)) + 
             pow(log(FA1(v(w),tl(s6),0,0,u(z),0)),2) + pow(log(FA2(v(w),tl(s6),0,0,u(z),0)),2) + pow(log(FB1(v(w),tl(s6),0,0,u(z),0)),2) + pow(log(FB2(v(w),tl(s6),0,0,u(z),0)),2)) + 
          4*pow(log(FB2(v(w),tl(s6),0,0,u(z),0)),3))*XCA00(w,s6,0,z,0) + 3*((8*pow(log(z),2) - 
             2*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),2)))*XCA01(w,s6,0,0,0)\
           + 2*(log(FA1(v(w),tl(s6),0,0,u(z),0)*FA2(v(w),tl(s6),0,0,u(z),0))*log(RegulA(0,z,w)) + log(FB1(v(w),tl(s6),0,0,u(z),0)*FB2(v(w),tl(s6),0,0,u(z),0))*log(RegulB(0,z,w)) - 
             2*log(z)*log(RegulA(0,z,w)*RegulB(0,z,w)) - 4*pow(log(z),2) + pow(log(FA1(v(w),tl(s6),0,0,u(z),0)),2) + pow(log(FA2(v(w),tl(s6),0,0,u(z),0)),2) + 
             pow(log(FB1(v(w),tl(s6),0,0,u(z),0)),2) + pow(log(FB2(v(w),tl(s6),0,0,u(z),0)),2))*XCA01(w,s6,0,z,0) - 4*log(z)*XCA02(w,s6,0,0,0) + 4*log(z)*XCA02(w,s6,0,z,0) + 
          log(RegulA(0,z,w)*RegulB(0,z,w))*XCA02(w,s6,0,z,0) + XCA03(w,s6,0,0,0) - XCA03(w,s6,0,z,0))))/12.;
}

//third integration is over b (c after rescaling)

double CAI3EM3c(double w, double s6, double c) {
	return 0;
}

double CAI3EM2c(double w, double s6, double c) {
	return (pow(c,-1)*((4*log(c) - log(FA1(v(w),tl(s6),0,0,0,0)*FA2(v(w),tl(s6),0,0,0,0)*FB1(v(w),tl(s6),0,0,0,0)*FB2(v(w),tl(s6),0,0,0,0)))*XCA00(w,s6,0,0,0) + 
       (-4*log(c) + log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c))) + 2*log(pow(1 + pow(c,2),2)))*XCA00(w,s6,0,0,c)))/4.;
}

double CAI3EM1c(double w, double s6, double c) {
	return (pow(c,-1)*(-4*(-2*log(c)*log(FA1(v(w),tl(s6),0,0,0,0)*FA2(v(w),tl(s6),0,0,0,0)*FB1(v(w),tl(s6),0,0,0,0)*FB2(v(w),tl(s6),0,0,0,0)) + 4*pow(log(c),2) + pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) + 
          pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),2))*XCA00(w,s6,0,0,0) + 
       2*(log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c)))*log(pow(1 + pow(c,2),2)) - 
          4*log(c)*log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c))*pow(1 + pow(c,2),2)) + 8*pow(log(c),2) + 
          2*(pow(log(FA1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FA2(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB2(v(w),tl(s6),0,0,0,b(c))),2)))*
        XCA00(w,s6,0,0,c) + 4*log(c)*XCA01(w,s6,0,0,0) - log(FA1(v(w),tl(s6),0,0,0,0)*FA2(v(w),tl(s6),0,0,0,0)*FB1(v(w),tl(s6),0,0,0,0)*FB2(v(w),tl(s6),0,0,0,0))*XCA01(w,s6,0,0,0) - 
       4*log(c)*XCA01(w,s6,0,0,c) + log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c)))*XCA01(w,s6,0,0,c) + 
       2*log(pow(1 + pow(c,2),2))*XCA01(w,s6,0,0,c)))/4.;
}

double CAI3E0c(double w, double s6, double c) {
	return (pow(c,-1)*(8*(-3*log(FA1(v(w),tl(s6),0,0,0,0)*FA2(v(w),tl(s6),0,0,0,0)*FB1(v(w),tl(s6),0,0,0,0)*FB2(v(w),tl(s6),0,0,0,0))*pow(log(c),2) + 4*pow(log(c),3) - 
          pow(log(FA1(v(w),tl(s6),0,0,0,0)),3) - pow(log(FA2(v(w),tl(s6),0,0,0,0)),3) - pow(log(FB1(v(w),tl(s6),0,0,0,0)),3) + 
          3*log(c)*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),2)) - 
          pow(log(FB2(v(w),tl(s6),0,0,0,0)),3))*XCA00(w,s6,0,0,0) - 4*(-2*(3*
              log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c))) + 2*log(pow(1 + pow(c,2),2)))*pow(log(c),2) + 
          8*pow(log(c),3) - log(pow(1 + pow(c,2),2))*(pow(log(FA1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FA2(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB1(v(w),tl(s6),0,0,0,b(c))),2) + 
             pow(log(FB2(v(w),tl(s6),0,0,0,b(c))),2)) + 2*log(c)*(log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c)))*
              log(pow(1 + pow(c,2),2)) + 3*(pow(log(FA1(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FA2(v(w),tl(s6),0,0,0,b(c))),2) + pow(log(FB1(v(w),tl(s6),0,0,0,b(c))),2) + 
                pow(log(FB2(v(w),tl(s6),0,0,0,b(c))),2))) - 2*(pow(log(FA1(v(w),tl(s6),0,0,0,b(c))),3) + pow(log(FA2(v(w),tl(s6),0,0,0,b(c))),3) + pow(log(FB1(v(w),tl(s6),0,0,0,b(c))),3) + 
             pow(log(FB2(v(w),tl(s6),0,0,0,b(c))),3)))*XCA00(w,s6,0,0,c) + 8*log(c)*log(FA1(v(w),tl(s6),0,0,0,0)*FA2(v(w),tl(s6),0,0,0,0)*FB1(v(w),tl(s6),0,0,0,0)*FB2(v(w),tl(s6),0,0,0,0))*
        XCA01(w,s6,0,0,0) - 16*pow(log(c),2)*XCA01(w,s6,0,0,0) - 4*pow(log(FA1(v(w),tl(s6),0,0,0,0)),2)*XCA01(w,s6,0,0,0) - 4*pow(log(FA2(v(w),tl(s6),0,0,0,0)),2)*XCA01(w,s6,0,0,0) - 
       4*pow(log(FB1(v(w),tl(s6),0,0,0,0)),2)*XCA01(w,s6,0,0,0) - 4*pow(log(FB2(v(w),tl(s6),0,0,0,0)),2)*XCA01(w,s6,0,0,0) + 
       2*log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c)))*log(pow(1 + pow(c,2),2))*XCA01(w,s6,0,0,c) - 
       8*log(c)*log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c))*pow(1 + pow(c,2),2))*XCA01(w,s6,0,0,c) + 
       16*pow(log(c),2)*XCA01(w,s6,0,0,c) + 4*pow(log(FA1(v(w),tl(s6),0,0,0,b(c))),2)*XCA01(w,s6,0,0,c) + 4*pow(log(FA2(v(w),tl(s6),0,0,0,b(c))),2)*XCA01(w,s6,0,0,c) + 
       4*pow(log(FB1(v(w),tl(s6),0,0,0,b(c))),2)*XCA01(w,s6,0,0,c) + 4*pow(log(FB2(v(w),tl(s6),0,0,0,b(c))),2)*XCA01(w,s6,0,0,c) + 4*log(c)*XCA02(w,s6,0,0,0) - 
       log(FA1(v(w),tl(s6),0,0,0,0)*FA2(v(w),tl(s6),0,0,0,0)*FB1(v(w),tl(s6),0,0,0,0)*FB2(v(w),tl(s6),0,0,0,0))*XCA02(w,s6,0,0,0) - 4*log(c)*XCA02(w,s6,0,0,c) + 
       log(FA1(v(w),tl(s6),0,0,0,b(c))*FA2(v(w),tl(s6),0,0,0,b(c))*FB1(v(w),tl(s6),0,0,0,b(c))*FB2(v(w),tl(s6),0,0,0,b(c)))*XCA02(w,s6,0,0,c) + 2*log(pow(1 + pow(c,2),2))*XCA02(w,s6,0,0,c)))
    /4.;
}

//third integration is over y (x after rescaling)

double CAI3EM3x(double w, double s6, double x) {
	return 0;
}

double CAI3EM2x(double w, double s6, double x) {
	return (mexp*log(FA1(v(w),tl(s6),0,0,0,0)*FA2(v(w),tl(s6),0,0,0,0)*FB1(v(w),tl(s6),0,0,0,0)*FB2(v(w),tl(s6),0,0,0,0)*pow(FA1(v(w),tl(s6),y(x),0,0,0),-1)*pow(FA2(v(w),tl(s6),y(x),0,0,0),-1)*
       pow(FB1(v(w),tl(s6),y(x),0,0,0),-1)*pow(FB2(v(w),tl(s6),y(x),0,0,0),-1))*pow(x,-1)*XCA00(w,s6,0,0,0))/8.;
}

double CAI3EM1x(double w, double s6, double x) {
	return (mexp*pow(x,-1)*(2*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) - pow(log(FA1(v(w),tl(s6),y(x),0,0,0)),2) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) - pow(log(FA2(v(w),tl(s6),y(x),0,0,0)),2) + 
          pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) - pow(log(FB1(v(w),tl(s6),y(x),0,0,0)),2) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),2) - pow(log(FB2(v(w),tl(s6),y(x),0,0,0)),2))*XCA00(w,s6,0,0,0)\
        + log(FA1(v(w),tl(s6),0,0,0,0)*FA2(v(w),tl(s6),0,0,0,0)*FB1(v(w),tl(s6),0,0,0,0)*FB2(v(w),tl(s6),0,0,0,0)*pow(FA1(v(w),tl(s6),y(x),0,0,0),-1)*pow(FA2(v(w),tl(s6),y(x),0,0,0),-1)*
          pow(FB1(v(w),tl(s6),y(x),0,0,0),-1)*pow(FB2(v(w),tl(s6),y(x),0,0,0),-1))*XCA01(w,s6,0,0,0)))/8.;
}

double CAI3E0x(double w, double s6, double x) {
	return (mexp*pow(x,-1)*(8*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),3) - pow(log(FA1(v(w),tl(s6),y(x),0,0,0)),3) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),3) - pow(log(FA2(v(w),tl(s6),y(x),0,0,0)),3) + 
          pow(log(FB1(v(w),tl(s6),0,0,0,0)),3) - pow(log(FB1(v(w),tl(s6),y(x),0,0,0)),3) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),3) - pow(log(FB2(v(w),tl(s6),y(x),0,0,0)),3))*XCA00(w,s6,0,0,0)\
        + 6*(pow(log(FA1(v(w),tl(s6),0,0,0,0)),2) - pow(log(FA1(v(w),tl(s6),y(x),0,0,0)),2) + pow(log(FA2(v(w),tl(s6),0,0,0,0)),2) - pow(log(FA2(v(w),tl(s6),y(x),0,0,0)),2) + 
          pow(log(FB1(v(w),tl(s6),0,0,0,0)),2) - pow(log(FB1(v(w),tl(s6),y(x),0,0,0)),2) + pow(log(FB2(v(w),tl(s6),0,0,0,0)),2) - pow(log(FB2(v(w),tl(s6),y(x),0,0,0)),2))*XCA01(w,s6,0,0,0)\
        + 3*log(FA1(v(w),tl(s6),0,0,0,0)*FA2(v(w),tl(s6),0,0,0,0)*FB1(v(w),tl(s6),0,0,0,0)*FB2(v(w),tl(s6),0,0,0,0)*pow(FA1(v(w),tl(s6),y(x),0,0,0),-1)*pow(FA2(v(w),tl(s6),y(x),0,0,0),-1)*
          pow(FB1(v(w),tl(s6),y(x),0,0,0),-1)*pow(FB2(v(w),tl(s6),y(x),0,0,0),-1))*XCA02(w,s6,0,0,0)))/24.;
}
